#version 150

#moj_import <fog.glsl>

#moj_import <map_data.glsl>

#define FRACTAL_OUTLINE_COLOR (vec3(1.0, 0.2, 0.4))
#define FRACTAL_BASE_COLOR (vec3(1.0, 0.2, 0.4))
#define FRACTAL_ITERATIONS (50.0)
#define FRACTAL_STATE (vec2(-1.4, 0.0))
#define FRACTAL_POWER (2.0)
#define FRACTAL_SCALE (vec2(5.0, 5.0))
#define FRACTAL_INFINITY_LIMIT (20.0)

uniform sampler2D Sampler0;

uniform vec4 ColorModulator;
uniform float FogStart;
uniform float FogEnd;
uniform vec4 FogColor;

uniform float GameTime;

in float vertexDistance;
in vec4 vertexColor;
in vec2 texCoord0;
in vec4 normal;

in vec3 chunkPosition;
in vec3 blockPosition;
in vec4 fractalPosition;
in vec4 baseVertexColor;

out vec4 fragColor;

void main() {
    vec4 color = texture(Sampler0, texCoord0);
	float fogStart = FogStart;
	float fogEnd = FogEnd;

	if (color.a >= 0.8 && color.a < 0.9) {
		if (chunkPosition.y < 0.0) {
			color *= vertexColor * ColorModulator;

			fogStart -= REPEAT_LENGTH;
			fogEnd -= REPEAT_LENGTH;
		}
		else {
			discard;
		}
	}
    else if (color.a >= 0.7 && color.a < 0.8) {
		if (blockPosition.z + chunkPosition.z < 0.0) {
		    vec2 uv = getScreenUv(fractalPosition);

			vec2 c = uv * FRACTAL_SCALE;

			float iter;
			for (iter = 0.0; iter < FRACTAL_ITERATIONS; iter++) {
				if (FRACTAL_POWER == 2.0) {
					c = vec2(c.x * c.x - c.y * c.y, 2.0 * c.x * c.y) + vec2(FRACTAL_STATE.x + sin(GameTime * (PI * 48.0)), FRACTAL_STATE.y);
				}
				else {
					c = vec2(pow(c.x * c.x + c.y * c.y, FRACTAL_POWER / 2) * cos(FRACTAL_POWER * atan(c.y, c.x)), pow(c.x * c.x + c.y * c.y, FRACTAL_POWER / 2) * sin(FRACTAL_POWER * atan(c.y, c.x))) + vec2(FRACTAL_STATE.x + sin(GameTime * (PI * 48.0)), FRACTAL_STATE.y);
				}

				if (length(c) > FRACTAL_INFINITY_LIMIT) {
					break;
				}
			}

			if (iter < FRACTAL_ITERATIONS) {
				color = vec4((iter / FRACTAL_ITERATIONS) * FRACTAL_OUTLINE_COLOR.r, (iter / FRACTAL_ITERATIONS) * FRACTAL_OUTLINE_COLOR.g, (iter / FRACTAL_ITERATIONS) * FRACTAL_OUTLINE_COLOR.b, 1.0);
			}
			else {
				color = vec4(FRACTAL_BASE_COLOR, 1.0);
			}
		}
		else {
			color = vec4(0.0, 0.0, 0.0, 1.0);
		}
    }
	else if (color.a >= 0.6 && color.a < 0.7) {
		if (chunkPosition.z < 16.0) {
			color *= baseVertexColor * ColorModulator;
		}
		else {
			discard;
		}
		
	}
    else if (!(color.a >= 0.5 && color.a < 0.6)) {
        color *= vertexColor * ColorModulator;
    }

	color.a = 1.0;

    fragColor = linear_fog(color, vertexDistance, fogStart, fogEnd, FogColor);
}
